@@ -5,15 +5,17 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1995-2003 Insecure.Com LLC. This       *
- * program is free software; you may redistribute and/or modify it under   *
- * the terms of the GNU General Public License as published by the Free    *
+ * The Nmap Security Scanner is (C) 1996-2004 Insecure.Com LLC. Nmap       *
+ * is also a registered trademark of Insecure.Com LLC.  This program is    *
+ * free software; you may redistribute and/or modify it under the          *
+ * terms of the GNU General Public License as published by the Free        *
  * Software Foundation; Version 2.  This guarantees your right to use,     *
  * modify, and redistribute this software under certain conditions.  If    *
  * you wish to embed Nmap technology into proprietary software, we may be  *
  * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as   *
- * our remote OS fingerprinting database and code.                         *
+ * Many security scanner vendors already license Nmap technology such as  *
+ * our remote OS fingerprinting database and code, service/version         *
+ * detection system, and port scanning code.                               *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -23,8 +25,11 @@
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
  *   nmap-os-fingerprints or nmap-service-probes.                          *
- * o Executes Nmap                                                         *
- * o Integrates/includes/aggregates Nmap into an executable installer      *
+ * o Executes Nmap and parses the results (as opposed to typical shell or  *
+ *   execution-menu apps, which simply display raw Nmap output and so are  *
+ *   not derivative works.)                                                * 
+ * o Integrates/includes/aggregates Nmap into a proprietary executable     *
+ *   installer, such as those produced by InstallShield.                   *
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
@@ -51,8 +56,17 @@
  * the continued development of Nmap technology.  Please email             *
  * sales@insecure.com for further information.                             *
  *                                                                         *
+ * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+ * permission to link the code of this program with any version of the     *
+ * OpenSSL library which is distributed under a license identical to that  *
+ * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * combinations including the two. You must obey the GNU GPL in all        *
+ * respects for all of the code used other than OpenSSL.  If you modify    *
+ * this file, you may extend this exception to your version of the file,   *
+ * but you are not obligated to do so.                                     *
+ *                                                                         *
  * If you received these files with a written license agreement or         *
- * contract stating terms other than the (GPL) terms above, then that      *
+ * contract stating terms other than the terms above, then that            *
  * alternative license agreement takes precedence over these comments.     *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
@@ -78,11 +92,12 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html .                                  *
+ * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
+ * with Nmap.                                                              *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: NmapOps.cc,v 1.21 2003/09/16 06:04:55 fyodor Exp $ */
+/* $Id: NmapOps.cc,v 1.29 2005/01/31 20:40:45 fyodor Exp $ */
 #include "nmap.h"
 #include "nbase.h"
 #include "NmapOps.h"
@@ -91,11 +106,13 @@ NmapOps o;
 
 NmapOps::NmapOps() {
   datadir = NULL;
+  xsl_stylesheet = NULL;
   Initialize();
 }
 
 NmapOps::~NmapOps() {
   if (datadir) free(datadir);
+  if (xsl_stylesheet) free(xsl_stylesheet);
 }
 
 void NmapOps::ReInit() {
@@ -154,6 +171,8 @@ int NmapOps::TimeSinceStartMS(struct timeval *now) {
 }
 
 void NmapOps::Initialize() {
+  char tmpxsl[MAXPATHLEN];
+
   setaf(AF_INET);
 #ifndef WIN32
 # ifdef __amigaos__
@@ -171,7 +190,7 @@ void NmapOps::Initialize() {
   spoofsource = 0;
   device[0] = '\0';
   interactivemode = 0;
-  host_group_sz = HOST_GROUP_SZ;
+  ping_group_sz = PING_GROUP_SZ;
   generate_random_ips = 0;
   reference_FPs = NULL;
   magic_port = 33000 + (get_random_uint() % 31000);
@@ -183,10 +202,13 @@ void NmapOps::Initialize() {
   max_rtt_timeout = MAX_RTT_TIMEOUT;
   min_rtt_timeout = MIN_RTT_TIMEOUT;
   initial_rtt_timeout = INITIAL_RTT_TIMEOUT;
+  min_host_group_sz = 1;
+  max_host_group_sz = 100000; // don't want to be restrictive unles user sets
+  max_tcp_scan_delay = MAX_TCP_SCAN_DELAY;
+  max_udp_scan_delay = MAX_UDP_SCAN_DELAY;
   max_ips_to_scan = 0;
   extra_payload_length = 0;
   extra_payload = NULL;
-  host_timeout = HOST_TIMEOUT;
   scan_delay = 0;
   scanflags = -1;
   resume_ip.s_addr = 0;
@@ -194,7 +216,6 @@ void NmapOps::Initialize() {
   osscan_guess = 0;
   numdecoys = 0;
   decoyturn = -1;
-  identscan = 0;
   osscan = 0;
   servicescan = 0;
   pingtype = PINGTYPE_UNKNOWN;
@@ -209,6 +230,10 @@ void NmapOps::Initialize() {
   pTrace = vTrace = false;
   if (datadir) free(datadir);
   datadir = NULL;
+
+  snprintf(tmpxsl, sizeof(tmpxsl), "%s/nmap.xsl", NMAPDATADIR);
+  if (xsl_stylesheet) free(xsl_stylesheet);
+  xsl_stylesheet = strdup(tmpxsl);
 }
 
 bool NmapOps::TCPScan() {
@@ -219,6 +244,19 @@ bool NmapOps::UDPScan() {
   return udpscan;
 }
 
+  /* this function does not currently cover cases such as TCP SYN ping
+     scan which can go either way based on whether the user is root or
+     IPv6 is being used.  It will return false in those cases where a
+     RawScan is not neccessarily used. */
+bool NmapOps::RawScan() {
+  if (ackscan|finscan|idlescan|ipprotscan|maimonscan|nullscan|osscan|synscan|udpscan|windowscan|xmasscan)
+    return true;
+  if (o.pingtype & (PINGTYPE_ICMP_PING|PINGTYPE_ICMP_MASK|PINGTYPE_ICMP_TS|PINGTYPE_TCP_USE_ACK|PINGTYPE_RAWTCP|PINGTYPE_UDP))
+    return true;
+
+   return false; 
+}
+
 
 void NmapOps::ValidateOptions() {
 
@@ -272,7 +310,7 @@ void NmapOps::ValidateOptions() {
  }
 
  if (ipprotscan + (TCPScan() || UDPScan()) + listscan + pingscan > 1) {
-   fatal("Sorry, the IPProtoscan, Listscan, and Pingscan (-sO, -sL, -sP) must currently be used alone rathre than combined with other scan types.");
+   fatal("Sorry, the IPProtoscan, Listscan, and Pingscan (-sO, -sL, -sP) must currently be used alone rather than combined with other scan types.");
  }
 
  if ((pingscan && pingtype == PINGTYPE_NONE)) {
@@ -346,13 +384,10 @@ void NmapOps::ValidateOptions() {
     error("WARNING: Decoys are irrelevant to the bounce or connect scans");
   }
   
-  if (fragscan && !(ackscan|finscan|maimonscan|nullscan|synscan|windowscan|xmasscan)) {
-    fatal("Fragscan only works with ACK, FIN, Maimon, NULL, SYN, Window, and XMAS scan types");
-  }
-  
-  if (identscan && !connectscan) {
-    error("Identscan only works with connect scan (-sT) ... ignoring option");
-    identscan = 0;
+  if (fragscan && !(ackscan|finscan|maimonscan|nullscan|synscan|windowscan|xmasscan) && \
+      !(pingtype&(PINGTYPE_ICMP_TS|PINGTYPE_TCP)) && !(fragscan == 8 && pingtype&PINGTYPE_ICMP_MASK) && \
+      !(extra_payload_length + 8 > fragscan)) {
+    fatal("Fragscan only works with TCP, ICMP Timestamp or ICMP Mask (mtu=8) ping types or ACK, FIN, Maimon, NULL, SYN, Window, and XMAS scan types");
   }
   
   if (osscan && bouncescan)
@@ -407,3 +442,26 @@ void NmapOps::setInitialRttTimeout(int rtt)
   if (rtt > max_rtt_timeout) max_rtt_timeout = rtt;  
   if (rtt < min_rtt_timeout) min_rtt_timeout = rtt;
 }
+
+void NmapOps::setMinHostGroupSz(unsigned int sz) {
+  if (sz > max_host_group_sz)
+    fatal("Minimum host group size may not be set to greater than maximum size (currently %d)\n", max_host_group_sz);
+  min_host_group_sz = sz;
+}
+
+void NmapOps::setMaxHostGroupSz(unsigned int sz) {
+  if (sz < min_host_group_sz)
+    fatal("Maximum host group size may not be set to less than the maximum size (currently %d)\n", min_host_group_sz);
+  if (sz <= 0)
+    fatal("Max host size must be at least 1");
+  max_host_group_sz = sz;
+}
+
+  /* Sets the Name of the XML stylesheet to be printed in XML output.
+     If this is never called, a default stylesheet distributed with
+     Nmap is used.  If you call it with NULL as the xslname, no
+     stylesheet line is printed. */
+void NmapOps::setXSLStyleSheet(char *xslname) {
+  if (xsl_stylesheet) free(xsl_stylesheet);
+  xsl_stylesheet = xslname? strdup(xslname) : NULL;
+}
